﻿using System;
using System.Collections.Generic;
using System.Linq;
using ComponentFramework.Core;
using ComponentFramework.Structures;
using ComponentFramework.Tools;
using MTV3D65;

namespace ComponentFramework.Components
{
    public class Keyboard : Component, IKeyboardService
    {
        const int SimultaneousKeys = 256; // Overkill, but prevents overflows
        readonly TV_KEYDATA[] keyBuffer = new TV_KEYDATA[SimultaneousKeys];

        // Using TimedButtonState's here would be more complex... and unnecessary?
        readonly Dictionary<CONST_TV_KEY, ButtonState> keyStates = new Dictionary<CONST_TV_KEY, ButtonState>(EnumComparer<CONST_TV_KEY>.Instance);

        public Keyboard(ICore core) : base(core) { }

        public ButtonState GetKeyState(CONST_TV_KEY key)
        {
            ButtonState state;
            if (!keyStates.TryGetValue(key, out state))
                state = ButtonState.Up;

            return state;
        }

        public IEnumerable<CONST_TV_KEY> Down
        {
            get { return from state in keyStates where state.Value == ButtonState.Down select state.Key; }
        }

        public IEnumerable<CONST_TV_KEY> Pressed
        {
            get { return from state in keyStates where state.Value == ButtonState.Pressed select state.Key; }
        }

        public IEnumerable<CONST_TV_KEY> Released
        {
            get { return from state in keyStates where state.Value == ButtonState.Released select state.Key; }
        }

        readonly List<CONST_TV_KEY> toRemove = new List<CONST_TV_KEY>();
        public override void Update(TimeSpan elapsedTime)
        {
            if (keyStates.Keys.Count > 0)
            {
                foreach (var key in keyStates.Keys)
                    switch (keyStates[key])
                    {
                        case ButtonState.Pressed:
                        case ButtonState.Released:
                            toRemove.Add(key);
                            break;
                    }

                if (toRemove.Count > 0)
                {
                    foreach (var key in toRemove)
                    {
                        if (keyStates[key] == ButtonState.Pressed)
                            keyStates[key] = ButtonState.Down;
                        else
                            keyStates.Remove(key);
                    }
                    toRemove.Clear();
                }
            }

            int keyCount = 0;
            InputEngine.GetKeyBuffer(keyBuffer, ref keyCount);
            for (int i = 0; i < keyCount; i++)
            {
                var keyData = keyBuffer[i];
                var key = (CONST_TV_KEY) keyBuffer[i].Key;
                var state = keyData.Pressed == 1 ? ButtonState.Pressed : keyData.Released == 1 ? ButtonState.Released : ButtonState.Up;

                keyStates.Remove(key);
                if (state != ButtonState.Up)
                    keyStates.Add(key, state);   
            }
        }
    }

    public interface IKeyboardService : IService
    {
        ButtonState GetKeyState(CONST_TV_KEY key);

        IEnumerable<CONST_TV_KEY> Down { get; }
        IEnumerable<CONST_TV_KEY> Pressed { get; }
        IEnumerable<CONST_TV_KEY> Released { get; }
    }
}