#include "Common.fxi"

float3x3 LeftFilter = { 0.0, 0.7, 0.3,
                        0.0, 0.0, 0.0,
                        0.0, 0.0, 0.0 };
                        
float3x3 RightFilter = { 0, 0, 0,
                         0, 1, 0,
                         0, 0, 1 };
                         
float RedGamma = 1.5;

texture LeftTexture : TEXTURE0;
sampler LeftSampler = sampler_state 
{
	Texture = (LeftTexture);
	MinFilter = Point;
	MagFilter = Point;
};

texture RightTexture : TEXTURE1;
sampler RightSampler = sampler_state 
{
	Texture = (RightTexture);
	MinFilter = Point;
	MagFilter = Point;
};

struct VS_INPUT 
{
	float4 Position : POSITION;
	float2 TexCoord : TEXCOORD0;
};

struct VS_OUTPUT 
{
	float2 TexCoord : TEXCOORD0;
};
#define PS_INPUT VS_OUTPUT

VS_OUTPUT VS(VS_INPUT IN, out float4 outPosition : POSITION) 
{
	outPosition = Resample(IN.Position);

	VS_OUTPUT OUT;
	OUT.TexCoord = IN.TexCoord;

	return OUT;
}

float4 PS(PS_INPUT IN) : COLOR 
{
	float3 left = tex2D(LeftSampler, IN.TexCoord).rgb;
	float3 right = tex2D(RightSampler, IN.TexCoord).rgb;

	float3 color = mul(LeftFilter, left) + mul(RightFilter, right);
	color.r = pow(color.r, 1 / RedGamma);
			  
	return float4(color, 1);
}

technique ShaderModel2
{
	pass Main 
	{
		AlphaBlendEnable = False;
		AlphaTestEnable = False;
	
		VertexShader = compile vs_2_0 VS();
		PixelShader  = compile ps_2_0 PS();    
	}
}