﻿using ComponentFramework.Core;
using MTV3D65;
using SlimDX;
using SlimDX.Direct3D9;

namespace Stereoscopy
{
    [AutoLoad]
    class Crosshair : Component
    {
        TVViewport viewport;

        public Crosshair(ICore core) : base(core)
        {
            Order = 20;
        }

        public override void Initialize()
        {
            viewport = Core.Engine.GetViewport();
        }

        public override void PostDraw()
        {
            if (!Renderer.ImportanceSampling) return;

            var halfViewportSize = new Vector2(viewport.GetWidth(), viewport.GetHeight()) / 2;
            var halfCrosshairSize = halfViewportSize / Renderer.FocusAreaFactor;

            Screen2DImmediate.Action_Begin2D();

            Screen2DImmediate.Draw_Line(halfViewportSize.X - halfCrosshairSize.X, halfViewportSize.Y,
                                        halfViewportSize.X + halfCrosshairSize.X, halfViewportSize.Y, new Color4(0.5f, 1, 1, 1).ToArgb());
            Screen2DImmediate.Draw_Line(halfViewportSize.X, halfViewportSize.Y - halfCrosshairSize.Y,
                                        halfViewportSize.X, halfViewportSize.Y + halfCrosshairSize.Y, new Color4(0.5f, 1, 1, 1).ToArgb());

            Screen2DImmediate.Draw_FilledEllipse(halfViewportSize.X, halfViewportSize.Y, halfCrosshairSize.X, halfCrosshairSize.Y, 32, new Color4(0.25f, 0.5f, 0.5f, 0.5f).ToArgb());
            Screen2DImmediate.Draw_Ellipse(halfViewportSize.X, halfViewportSize.Y, halfCrosshairSize.X, halfCrosshairSize.Y, 32, new Color4(0.75f, 1, 1, 1).ToArgb());
            Screen2DImmediate.Draw_FilledBox(halfViewportSize.X - 1, halfViewportSize.Y - 1, halfViewportSize.X + 1, halfViewportSize.Y + 1);

            Device.SetRenderState(RenderState.AntialiasedLineEnable, true);
            Screen2DImmediate.Action_End2D();
            Device.SetRenderState(RenderState.AntialiasedLineEnable, false);
        }

        [ServiceDependency]
        IRendererService Renderer { get; set; }
    }
}
