﻿using System;
using ComponentFramework.Components;
using ComponentFramework.Core;
using ComponentFramework.Structures;
using ComponentFramework.Tools;
using MTV3D65;

namespace Stereoscopy
{
    [AutoLoad]
    class Settings : Component
    {
        public Settings(ICore core) : base(core)
        {
        }

        public override void Initialize()
        {
            DebuggingBag.Shadows = true;
        }

        public override void Update(TimeSpan elapsedTime)
        {
            if (Mouse.WheelTurns > 0)
                Camera.AngleOfView = Math.Min(Camera.AngleOfView + 5, 175);
            if (Mouse.WheelTurns < 0)
                Camera.AngleOfView = Math.Max(Camera.AngleOfView - 5, 5);

            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_1) == ButtonState.Down)
                Renderer.EyeSeparation += 0.001f;
            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_F1) == ButtonState.Down)
                Renderer.EyeSeparation = Math.Max(Renderer.EyeSeparation - 0.001f, 0.001f);

            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_2) == ButtonState.Pressed)
                Renderer.AutoFocus = true;
            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_F2) == ButtonState.Pressed)
                Renderer.AutoFocus = false;

            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_3) == ButtonState.Pressed)
                Renderer.ImportanceSampling = true;
            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_F3) == ButtonState.Pressed)
                Renderer.ImportanceSampling = false;

            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_4) == ButtonState.Pressed)
                Renderer.FocusAreaFactor = Math.Min(Renderer.FocusAreaFactor + 1, 16);
            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_F4) == ButtonState.Pressed)
                Renderer.FocusAreaFactor = Math.Max(Renderer.FocusAreaFactor - 1, 1);

            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_5) == ButtonState.Pressed)
                Renderer.RedGamma = Math.Min(Renderer.RedGamma + 0.1f, 2);
            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_F5) == ButtonState.Pressed)
                Renderer.RedGamma = Math.Max(Renderer.RedGamma - 0.1f, 0.5f);

            DebuggingBag.Put("Focal plane distance", Math.Round(Camera.FocalPlane, 2) + " m");
            DebuggingBag.Put("..", "..");
            DebuggingBag.Put("Angle of view [Mousewheel]", Math.Round(Camera.AngleOfView, 1) + "°");
            DebuggingBag.Put("Eye separation [1/F1]", Math.Round(Renderer.EyeSeparation * 100, 1) + "cm");
            DebuggingBag.Put("Auto-focus [2/F2]", Renderer.AutoFocus);
            DebuggingBag.Put("Focus importance sampling [3/F3]", Renderer.ImportanceSampling);
            DebuggingBag.Put("Focus area factor [4/F4]", "1/" + Renderer.FocusAreaFactor + " of screen");
            DebuggingBag.Put("Red gamma correction [5/F5]", Math.Round(Renderer.RedGamma, 1));
            DebuggingBag.Put("..", "..");

            if (Keyboard.GetKeyState(CONST_TV_KEY.TV_KEY_RETURN) == ButtonState.Pressed)
            {
                Camera.RevertToDefaults();
                Renderer.RevertToDefaults();
            }

            DebuggingBag.Put("Press Return", "to revert to default values");
        }

        [ServiceDependency]
        IKeyboardService Keyboard { get; set; }
        [ServiceDependency]
        IMouseService Mouse { get; set; }
        [ServiceDependency]
        IDebuggingBagService DebuggingBag { get; set; }
        [ServiceDependency]
        ICameraService Camera { get; set; }
        [ServiceDependency]
        IRendererService Renderer { get; set; }
    }
}
