﻿using System;
using ComponentFramework.Core;
using ComponentFramework.Tools;
using MTV3D65;
using SlimDX;

namespace Stereoscopy
{
    [AutoLoad]
    class World : Component
    {
        TVMesh floor;
        TVMesh teapot;
        TVMesh sky;

        public World(ICore core) : base(core) { }

        public override void Initialize()
        {
            LightEngine.SetGlobalAmbient(0, 0, 0);

            var light = new TV_LIGHT
            {
                direction = Vector3.Normalize(new Vector3(0, -1, 0.5f)).ToTvVector(),
                diffuse = new TV_COLOR(1, 1, 1, 1),
                ambient = new TV_COLOR(1, 1, 1, 1),
                range = 1000,
                type = CONST_TV_LIGHTTYPE.TV_LIGHT_DIRECTIONAL
            };
            LightEngine.EnableLight(LightEngine.CreateLight(ref light), true);

            var materialId = MaterialFactory.CreateMaterial();
            MaterialFactory.SetEmissive(materialId, 0, 0, 0, 1);
            MaterialFactory.SetDiffuse(materialId, 0.9f, 0.9f, 0.9f, 1);
            MaterialFactory.SetAmbient(materialId, 0.1f, 0.1f, 0.1f, 1);
            MaterialFactory.SetSpecular(materialId, 1, 1, 1, 1);
            MaterialFactory.SetOpacity(materialId, 1);

            teapot = Scene.CreateMeshBuilder();
            teapot.CreateTeapot();
            teapot.SetLightingMode(CONST_TV_LIGHTINGMODE.TV_LIGHTING_MANAGED);
            teapot.SetMaterial(materialId);

            floor = Scene.CreateMeshBuilder();
            floor.AddFloorGrid(-1, -25, -25, 25, 25, 10, 10, -1, 10, 10);
            floor.ComputeNormals();
            floor.SetLightingMode(CONST_TV_LIGHTINGMODE.TV_LIGHTING_MANAGED);
            floor.SetMaterial(materialId);
            floor.SetTexture(TextureFactory.LoadTexture("checkerboard.png"));

            sky = Scene.CreateMeshBuilder();
            sky.AddWall(TextureFactory.LoadTexture("Clouds_NegZ.dds"), -0.5f, -0.5f, 0.5f, -0.5f, 0.5f - -0.5f, -0.5f, 1, 1, true);
            sky.AddWall(TextureFactory.LoadTexture("Clouds_PosZ.dds"), 0.5f, 0.5f, -0.5f, 0.5f, 0.5f - -0.5f, -0.5f, 1, 1, true);
            sky.AddWall(TextureFactory.LoadTexture("Clouds_NegX.dds"), -0.5f, 0.5f, -0.5f, -0.5f, 0.5f - -0.5f, -0.5f, 1, 1, true);
            sky.AddWall(TextureFactory.LoadTexture("Clouds_PosX.dds"), 0.5f, -0.5f, 0.5f, 0.5f, 0.5f - -0.5f, -0.5f, 1, 1, true);
            sky.AddFloor(TextureFactory.LoadTexture("Clouds_NegY.dds"), -0.5f, 0.5f, 0.5f, -0.5f, -0.5f, -1, 1, true);
            sky.AddFloor(TextureFactory.LoadTexture("Clouds_PosY.dds"), -0.5f, -0.5f, 0.5f, 0.5f, 0.5f, -1, 1, true);
            sky.SetScale(Camera.FarPlane, Camera.FarPlane, Camera.FarPlane);
            sky.SetCullMode(CONST_TV_CULLING.TV_FRONT_CULL);

            Scene.SetTextureFilter(CONST_TV_TEXTUREFILTER.TV_FILTER_ANISOTROPIC);
            Scene.SetMaxAnisotropy(DeviceInfo.GetMaxAnisotropy());
        }

        public override void Update(TimeSpan elapsedTime)
        {
            var camPosition = Camera.Position;
            sky.SetPosition(camPosition.X, camPosition.Y, camPosition.Z);
        }

        public override void Draw()
        {
            if (!Renderer.PreRender) return;

            teapot.Render();
            floor.Render();
            sky.Render();
        }

        [ServiceDependency]
        IRendererService Renderer { get; set; }
        [ServiceDependency]
        ICameraService Camera { get; set; }
    }
}
